#!/usr/bin/env python3
"""
Example app that does nothing. You can select the font
from the File menu, or zoom via the View menu.
"""

import argparse, sys

from   uqtie           import UqtWin
from   PyQt5.QtWidgets import QApplication, QVBoxLayout

class TestAppMainWindow(UqtWin.MainWindow):

    def __init__(self, parsedArgs, **kwargs ):
        super(TestAppMainWindow, self).__init__(parsedArgs, **kwargs)
        self.setup_ui()
        self.show()

    def setup_ui(self):
        vbox = QVBoxLayout(self.centralWidget())

        # Here you can add widgets to the vbox layout
        # ... etc.

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-x', '--test', help='Test Argument placeholder', default='Test')
    parsedArgs,unparsedArgs = parser.parse_known_args()

    # Pass unparsed args to Qt, might have some X Windows args, like --display
    qtArgs = sys.argv[:1] + unparsedArgs
    app = QApplication(qtArgs)

    # can add optional title='OptionalTitle' if it is different from your app name
    mainw = TestAppMainWindow(parsedArgs, app=app, organizationName='Craton', appName='UqtTest')

    sys.exit(app.exec_())



#!/usr/bin/env python
"""
Classes for using pymavlink and mavutil from within a PyQt application
"""

from __future__   import print_function

from pymavlink    import mavutil
from argparse     import ArgumentParser
from builtins     import object
from PyQt5.QtCore import QThread, QObject, pyqtSignal, pyqtSlot, QSocketNotifier

import time, sys, os

# To be addressed:
#   how to detach from QSocketNotifier
#   how to identify socket error conditions, since they happen in mavutil
#   how to convey the timestamps from the file (when reading a file)

class UqtMavFileRecvWorker(QObject):
    """Worker thread for reading from a file and emitting MAVLink messages as Qt Signals"""

    finished = pyqtSignal()

    def __init__(self, mavfile, rxMsgSignal, msg_types=None ):
        super(self.__class__, self).__init__()

        self.msg_types = msg_types
        self.mavfile = mavfile
        self.rxMsgSignal = rxMsgSignal

    @pyqtSlot()
    def worker(self):
        """This is the actual thread service loop"""
        self.running = True
        nRx = 0
        while self.running:
            try:
                msg = self.mavfile.recv_msg()
                if msg:
                    self.rxMsgSignal.emit(msg, time.time())
                    nRx += 1
                else:
                    break
            except Exception as error:
                print(f'{error}' )
                #self.running = False
                #break

        self.rxMsgSignal.emit(None, 0)
        self.finished.emit()

class UqtMavConn(object):
    '''
    Container for MAVLink connection objects provided by pymavlink and mavutil

    In mavutil, there is a class called mavfile. Several classes are derived
    from it:
       - mavudp
       - mavtcp
       - mavtcpin
       - mavserial
       - mavmcast
       - mavlogfile
       - mavchildexec

    mavfile *has* a mavlink, so all the mavfiles have a protocol object that is
    the exact same class. So mavutil.mavfile (a connection) has a mavlink.mavlink
    (a protocol - or rather a dialect, if you are familiar with mavlink)

    mavutil.mavlink_connection() is the factory for mavfiles.

    That's all pretty good, and not neccessary to improve upon. However, I
    want to eliminate some of the cut-and-paste I do when putting together
    apps, and also create a pattern for usage within Qt. So that's what this 
    is.

    So, UqtMavConn is an object that has a mavfile. It has methods for:

    - providing command line args so the client app can make those available
    - managing receive scheduling in the Qt environment

    Client is a Qt app with a UqtMavConn, and wants to receive in the Qt main loop
    via a signal. We will provide this service either via a worker thread or a
    QSocketNotifier, as dictated by the characteristics of the underlying mavfile.
    '''
    def __init__(self, cmdLineArgs=None, rxMsgSignal=None, recvMatch=None):
        self.mavfile = None

        self.rxMsgSignal = None
        self.rxMsgSignal = rxMsgSignal
        self.recvMatch = recvMatch
        self.notifier = None

        portStr = None
        dialectStr = None
        baud = None
        serial = False
        if cmdLineArgs:
            if cmdLineArgs.mavlink_version:
                # mavutil reads os.environ instead of taking args for some things
                if cmdLineArgs.mavlink_version == '1.0':
                    del(os.environ['MAVLINK20'])
                elif cmdLineArgs.mavlink_version == '2.0':
                    os.environ['MAVLINK20'] = '1'

            if cmdLineArgs.mavlink_dialect:
                dialectStr = cmdLineArgs.mavlink_dialect

            if cmdLineArgs.mavlink_port:
                portStr = cmdLineArgs.mavlink_port
                if 'serial:' in portStr:
                    serial = True
                    portStr = portStr[7:]
                    commaIx = portStr.find(',') 
                    if commaIx > 0:
                        baudStr = portStr[commaIx+1:]
                        baud = int(baudStr)
                        portStr = portStr[:commaIx]
                elif 'file:' in portStr:
                    portStr = portStr[5:]

        if portStr:
            if dialectStr:
                os.environ['MAVLINK_DIALECT'] = dialectStr
            
            try:
                # create a mavlink connection
                if serial and baud:
                    self.mavfile = mavutil.mavlink_connection(portStr, baud=baud, dialect=dialectStr )
                else:
                    self.mavfile = mavutil.mavlink_connection(portStr, dialect=dialectStr )
            except IOError:
                pass

        if self.mavfile:
            # save self.mavfile.mav as self.mavlink so users can do:
            #  mavConn = UqtMavConn(args)
            #  mavConn.mavlink.arbitrary_mavlink_method()
            self.mavlink = self.mavfile.mav
            print ( "MAVLink instantiation successful" )

            # Set up receive handling
            self.register_qt_recv_signal(self.rxMsgSignal, recvMatch=self.recvMatch)

    def register_qt_recv_signal(self, rxMsgSignal, recvMatch=None):
        """The client is a Qt app, and wants to receive the messages specified in recvMatch
         via the rxMsgSignal
        """
        self.rxMsgSignal = rxMsgSignal
        self.recvMatch = recvMatch

        if self.mavfile:
            if isinstance(self.mavfile, mavutil.mavmmaplog):
                self.thread = QThread()
                self.recvWorker = UqtMavFileRecvWorker(self.mavfile, self.rxMsgSignal, self.recvMatch)
                self.recvWorker.moveToThread(self.thread)
                self.recvWorker.finished.connect(self.thread.quit)                
                self.thread.started.connect(self.recvWorker.worker)
                self.thread.start()

            elif type(self.mavfile) in [ mavutil.mavserial ]:
                fd = self.get_file_descriptor()
                if fd >= 0:
                    self.notifier = QSocketNotifier(fd, QSocketNotifier.Read)
                    self.notifier.activated.connect(self.can_read)

            elif type(self.mavfile) in [ mavutil.mavudp, mavutil.mavtcp, mavutil.mavtcpin,
                                         mavutil.mavmcast, mavutil.mavlogfile, mavutil.mavlogfile ]:
                fd = self.get_file_descriptor()
                if fd:
                    self.notifier = QSocketNotifier(fd.fileno(), QSocketNotifier.Read)
                    self.notifier.activated.connect(self.can_read)
            elif isinstance(self.mavfile, mavutil.mavchildexec):
                pass

    def can_read(self):
        try:
            msg = self.mavfile.recv_msg()
            if msg:
                self.rxMsgSignal.emit(msg, time.time())
            else:
                self.rxMsgSignal.emit(None, 0)

        except Exception as error:
            print(f'{__name__}: {error}' )
            self.rxMsgSignal.emit(None, 0)

    def get_file_descriptor(self):
        '''
        return the file descriptor of the mavutil.mavfile for use in select()
        '''
        fileDescriptor = -1
        if self.mavfile:
            if isinstance(self.mavfile, mavutil.mavudp):
                fileDescriptor = self.mavfile.port
            elif isinstance(self.mavfile, mavutil.mavtcp):
                fileDescriptor = self.mavfile.port
            elif isinstance(self.mavfile, mavutil.mavtcpin):
                fileDescriptor = self.mavfile.port
            elif isinstance(self.mavfile, mavutil.mavserial):
                fileDescriptor = self.mavfile.port.fileno()
            elif isinstance(self.mavfile, mavutil.mavmcast):
                fileDescriptor = self.mavfile.port
            elif isinstance(self.mavfile, mavutil.mavlogfile):
                fileDescriptor = self.mavfile.port
            elif isinstance(self.mavfile, mavutil.mavchildexec):
                fileDescriptor = self.mavfile.port

        return fileDescriptor

    @staticmethod
    def make_args():
        '''
        Return an args to be used as 'parent' in another ArgumentParser
        These args are for UqtMavConn to use
        '''
        mavArgParser = ArgumentParser(add_help=False)

        '''
        TBD: the mavlink-port arg needs to support everything (?)that mavutil.mavlink_connection() supports
        def mavlink_connection(device, baud=115200, source_system=255, source_component=0,
                               planner_format=None, write=False, append=False,
                               robust_parsing=True, notimestamps=False, input=True,
                               dialect=None, autoreconnect=False, zero_time_base=False,
                               retries=3, use_native=default_native,
                               force_connected=False, progress_callback=None):
        '''

        help = 'serial:DEV[,BAUD] | udp:IP:PORT | tcp:IP:PORT | file:FILE | auto.\
                Example: --mavlink-port serial:/dev/ttyUSB0,9600'
        mavArgParser.add_argument("--mavlink-port", metavar='PORT', help=help)

        mavArgParser.add_argument("--mavlink-dialect", metavar='DIALECT',
                                   help='Example: --mavlink-dialect standard')

        mavArgParser.add_argument("--mavlink-version", metavar='VERSION',
                                   help='Example: --mavlink-version 2.0')
        return mavArgParser

# Test Code
def main():

    mavArgParser = UqtMavConn.make_args()
    
    parser = ArgumentParser(parents=[mavArgParser])
    parser.add_argument('-x', '--test', help='Test Argument placeholder', default='Test')
    parsedArgs,unparsedArgs = parser.parse_known_args()

    conn = UqtMavConn(cmdLineArgs=parsedArgs)
    if conn:
        nMessages = 0
        while True:
            m = conn.mavfile.recv_match()

            # If there's no match, it means we're done processing the log.
            if m is None:
                break

            # Ignore any failed messages
            if m.get_type() == 'BAD_DATA':
                continue
            nMessages += 1

        print ( "RX {n} messages".format(n=nMessages))

if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
PyQt5 Application Main Window base class that does basic things that most of my GUI apps need.
"""
from __future__      import print_function
from pathlib         import Path

import sys, os, argparse, time

from PyQt5.QtCore    import Qt, QSettings
from PyQt5.QtGui     import QKeySequence, QFont, QFontInfo
from PyQt5.QtWidgets import QWidget, QApplication, QAction, QMainWindow, QShortcut, QFontDialog
from uqtie           import UqtStylesheet

class MainWindow(QMainWindow):

    def __init__(self, parsedArgs, **kwargs ):
        super(MainWindow, self).__init__()

        self.fontDialog = None
        self.stylesheetFileName = None

        # Save the app so we can change its stylesheet
        if 'app' in kwargs:
            self.app = kwargs['app']

        # Create a QSettings. Qt stores this using your organization name and your application name
        # to determine the correct place for your user and their operating system
        # (e.g. on linux it's: ~/.config/Craton/SomeApp)
        if 'organizationName' in kwargs and 'appName' in kwargs:
            self.settings = QSettings(kwargs['organizationName'], kwargs['appName'])

            if self.settings and self.app:
                self.stylesheetMgr = UqtStylesheet.StylesheetManager(self.app, self.settings,
                                                                     kwargs['appName'])

            if self.stylesheetMgr:
                self.stylesheetMgr.read_stylesheet_vars()
                self.stylesheetMgr.apply()

        # stylesheet inspector is cool but needs improvement. Need to use the branch that has
        # the tree view. Need to make the inspector window re-sizable and use our font, so it
        # isn't ridiculously small or ridiculously large.
        self.connect_stylesheet_inspector(shortcut=QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_F12))

        appName = ''
        title = ''
        if 'appName' in kwargs:
            appName = kwargs['appName']
        if 'title' in kwargs:
            title = kwargs['title']

        if title == '':
            title = appName

        self.setup_base_ui( title )

    def connect_stylesheet_inspector(self, shortcut):
        self.shortcut = shortcut
        shortcut_ = QShortcut(self.shortcut, self)
        shortcut_.setContext(Qt.ApplicationShortcut)
        shortcut_.activated.connect(self.show_stylesheet_editor)

    def show_stylesheet_editor(self):
        style_sheet_inspector_class = self.get_stylesheet_inspector_class()
        style_sheet_inspector = [
            c for c in self.children() if
            isinstance(c, style_sheet_inspector_class)]
        if style_sheet_inspector:
            style_sheet_inspector = style_sheet_inspector[0]
        else:
            style_sheet_inspector = style_sheet_inspector_class(self)
            style_sheet_inspector.setFixedSize(800, 600)
        style_sheet_inspector.show()

    def get_stylesheet_inspector_class(self):
        """
        Indirection mostly to simplify tests.
        """
        try:
            from qt_style_sheet_inspector import StyleSheetInspector
        except ImportError as error:
            msg = 'You need to Install qt_style_sheet_inspector.'
            raise RuntimeError(msg)
        return StyleSheetInspector


    def setup_base_menus(self):
        # Create Menu Bar
        self.menuBar = self.menuBar()

        # Create Root Menus
        self.fileMenu = self.menuBar.addMenu('File')
        self.viewMenu = self.menuBar.addMenu('View')

        # Set up our handling of file menu
        self.fileMenu.triggered.connect(self.file_menu_selected)

        # Create Actions for menus
        self.quitAction = QAction('Quit', self)
        self.quitAction.setShortcut('Ctrl+Q')
        self.quitAction.triggered.connect(self.onQuit)

        # Should go in File->Preferences... ? Or View?
        self.fontAction = QAction('Font', self)
        self.fontAction.triggered.connect(self.on_font_menu_selected)

        self.zoomInAction = QAction('Zoom In', self)
        self.zoomInAction.setShortcut('Ctrl++')
        self.zoomInAction.triggered.connect(self.on_zoom_in)

        self.zoomOutAction = QAction('Zoom Out', self)
        self.zoomOutAction.setShortcut('Ctrl+-')
        self.zoomOutAction.triggered.connect(self.on_zoom_out)

        # Add actions to Menus
        self.fileMenu.addAction(self.quitAction)
        self.fileMenu.addAction(self.fontAction)
        self.viewMenu.addAction(self.zoomInAction)
        self.viewMenu.addAction(self.zoomOutAction)

        # Events
        #self.fileMenu.triggered.connect(self.selected)


    def setup_base_ui(self, title ):
        self.setup_base_menus()
        centralWindow = QWidget()
        self.setCentralWidget(centralWindow)

        # Restore the geometry from settings, default to an empty string
        geometry = self.settings.value('geometry', '')
        if geometry:
            self.restoreGeometry(geometry)

        self.setWindowTitle(title)

    def on_font_menu_selected(self):
        """User has selected 'Font...' from an app menu, put up font dialog"""

        # Clean up from Previous Font Dialog.
        self.disconnect_font_dialog()
        self.fontDialog = None

        self.fontDialog = QFontDialog(QFont())

        # if you don't do this, .font() may not give you the widget's actual font
        self.fontDialog.ensurePolished()
        font = self.fontDialog.font()
        #print ("Dialog Font Post-Polish")
        #print (font.family())
        #print (font.pointSize())
        #print (font.pixelSize())

        fontinfo = QFontInfo(font)
        #print ('fontinfo.family(): {s}'.format(s=fontinfo.family()))
        #print ('fontinfo.styleName(): {s}'.format(s=fontinfo.styleName()))

        # Resize the Dialog Box so it is reasonable for the actual current font
        # (Not sure how valid this ratio is for different systems, works nicely on my
        # Cygwin/X setup on which I have a large 4K monitor on which I use large fonts)
        self.fontDialog.resize( fontinfo.pointSize() * 40, fontinfo.pointSize() * 40 )

        self.fontDialog.setCurrentFont(font)
        self.fontDialog.currentFontChanged.connect(self.on_font_dialog_current_font_changed)
        self.fontDialog.accepted.connect(self.on_font_dialog_accepted)
        self.fontDialog.finished.connect(self.on_font_dialog_finished)
        self.fontDialog.rejected.connect(self.on_font_dialog_rejected)

        self.fontDialog.show()

    def disconnect_font_dialog(self):
        if self.fontDialog:
            self.fontDialog.currentFontChanged.disconnect()
            self.fontDialog.accepted.disconnect()
            self.fontDialog.finished.disconnect()
            self.fontDialog.rejected.disconnect()
        
    def on_font_dialog_accepted(self):
        #print ( "on_font_dialog_accepted")

        # The way I read the docs, this is what I should call:
        #font = self.fontDialog.selectedFont()

        # But... this is what gives me the actual user selection in practice:
        font = self.fontDialog.currentFont()
        #print ('currentFont(): {s}'.format(s=str(font)))
        #print ('font.family(): {s}'.format(s=font.family()))
        fontinfo = QFontInfo(font)
        #print ('fontinfo.family(): {s}'.format(s=fontinfo.family()))
        #print ('fontinfo.styleName(): {s}'.format(s=fontinfo.styleName()))

        self.stylesheetMgr.set_main_font_family(font.family())
        #self.stylesheetMgr.set_main_font_weight(fontinfo.styleName())
        self.stylesheetMgr.set_main_font_size(str(font.pointSize())+ 'pt')
        
        self.stylesheetMgr.apply()
        self.stylesheetMgr.save_stylesheet_vars()

    def on_font_dialog_finished(self):
        #print ('on_font_dialog_finished')
        # On Cygwin/X I get the finished event before the accepted, so I can't do this:
        #self.disconnect_font_dialog()
        #self.fontDialog = None
        pass

    def on_font_dialog_rejected(self):
        #print ('on_font_dialog_rejected')
        pass

    def on_font_dialog_current_font_changed(self, font):
        #print ('on_font_dialog_current_font_changed')        
        #print (font.family())
        pass

    def on_zoom_in(self):
        self.stylesheetMgr.zoom_in()
        self.stylesheetMgr.apply()
        self.stylesheetMgr.save_stylesheet_vars()

    def on_zoom_out(self):
        self.stylesheetMgr.zoom_out()
        self.stylesheetMgr.apply()
        self.stylesheetMgr.save_stylesheet_vars()

    def save_geometry_on_quit(self):
        """Get and save the geometry (size, aspect ratio, position) of the main window"""
        geometry = self.saveGeometry()
        self.settings.setValue('geometry', geometry)
        #print ( 'save_geometry_on_quit {g}'.format(g=geometry) )
        self.settings.sync()

    def onQuit(self):
        self.save_geometry_on_quit()
        self.app.quit()

    def closeEvent(self, event):
        """
        This overridden method is called whenever a window is closed when you hit the upper right X
        on the window, call the same function we call for handling CTRL-Q or File Menu->Quit
        """
        self.save_geometry_on_quit()

        # Pass the event to the class we inherit from
        super(MainWindow, self).closeEvent(event)

    def file_menu_selected(self, q):
        #print(q.text() + ' selected')
        if q.text() == 'Save':
            if self.currentFileName:
                # just perform the save
                self.saveFileDialog()
            else:
                # put up the Save file dialog
                self.saveFileDialog()
                pass
        elif q.text() == 'New':
            # What does New mean?
            pass
        elif q.text() == 'Save As':
            # Similar to New I guess
            pass
        elif q.text() == 'Open':
            # Put up the Open File Dialog
            pass

## Test Code
class TestAppMainWindow(MainWindow):
    """Class that demonstrates deriving a class from MainWindow"""
    def __init__(self, parsedArgs, **kwargs ):
        super(TestAppMainWindow, self).__init__(parsedArgs, **kwargs)
        self.show()

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-x', '--test', help='Test Argument placeholder', default='Test')
    parsedArgs,unparsedArgs = parser.parse_known_args()

    # Pass unparsed args to Qt, might have some X Windows args, like --display
    qtArgs = sys.argv[:1] + unparsedArgs
    app = QApplication(qtArgs)

    mainw = TestAppMainWindow(parsedArgs, app=app, organizationName='Craton', appName='UqtWinTest',
                              title='Main Window' )

    sys.exit(app.exec_())




#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Class that manages a UQtie application's stylesheet

There are advantages and disadvantages to Qt stylesheets, Qt settings, and
Qt Style. They aren't mutually exclusive, and they don't all play together
either.

This module attempts to make it possible to use a stylesheet while still
using the QFontDialog to select fonts, and zoom-in/zoom-out shortcuts such
as CTRL-+.

The idea is that we have variables (which come from somewhere, right now a
pickled dictionary, but later could be elsewhere, or multiple elsewheres).
These variables can be used in the stylesheet, such that the dynamic changes in
appearance can be made at runtime without requiring stylesheet changes, and the
changes can be persisted also without changing the stylesheet.

Font, font size and widget sizes (e.g QScrollBar:vertical { width } ) seem like
good candidates to be determined dynamically instead of via hardcoded values
in a stylesheet. That way, when you have high-DPI monitor and less-than-perfect
vision, you can adjust easily.

Part of my motivation for doing this is because PyQt running on Windows, Linux,
MacOS and Cygwin doesn't behave identically, not even close to identical in
some ways. If they would all have identical QScreens given identical monitors
and graphics cards, then you could just make reasonable choices for your stylesheet,
and rely on the OS GUI settings.

Variables you can use in a QSS file are:

 $main_font_family
 $main_font_weight (not supported yet)
 $main_font_size
 $scroll_bar_width

Futures:
1) Add more variables
2) Make an inspector widget for stylesheet variables

"""

from   __future__      import print_function
from   pathlib         import Path

import os,re,pickle

from   PyQt5.QtCore    import QSettings

class StylesheetManager(object):
    """
    Class that manages a UQtie application's stylesheet
    """

    # If no stylesheet has been provided, use this one. Should this
    # really have  scroll bar dimensions?
    defaultStylesheet = """
    QWidget {
        font-family:   $main_font_family;
        font-weight:   $main_font_weight;
        font-size:     $main_font_size;
    }
    
    QScrollBar:vertical {
        width:         $scroll_bar_width;
    }
    
    QScrollBar:horizontal {
        height:        $scroll_bar_width;
    }
    """
    def __init__(self, app, settings, appName ):

        self.stylesheetFileName     = None
        self.stylesheetVarsFileName = None
        self.app                    = app
        self.appName                = appName
        self.appSettings            = settings
        self.varsDict               = {}

        self.defaultVarsDict        = {
            'main_font_family': 'Arial',
            'main_font_weight': 'Regular',
            'main_font_size'  : '16pt',
            'scroll_bar_width': '15px',
        }

        self.determine_stylesheet_filenames(appName)

    def determine_stylesheet_path(self):
        """
        Fill in self.appDirPath appropriately
        """
        self.appDirPath = None
        if os.name == 'nt':
            # running on Windows
            appDirPath = Path(os.path.expanduser('~')) / 'Application Files'
            if not appDirPath.is_dir():
                print ( '{p} is not a directory'.format(p=appDirPath))
                return

            appDirPath /= self.appName
            if not appDirPath.is_dir():
                try:
                    appDirPath.mkdir()
                except:
                    print ( 'Could not create directory {p}'.format(p=appdirPath))
                    return
            self.appDirPath = appDirPath
        else:
            # On other OS, we use Settings to determine where stylesheet lives
            if self.appSettings:
                self.appDirPath = Path(os.path.dirname(self.appSettings.fileName()))


    def determine_stylesheet_filenames(self, appName):
        """
        Fill in stylesheet filenames appropriately
        """
        self.determine_stylesheet_path()
        if self.appDirPath:
            #print ("self.appDirPath: {}".format(self.appDirPath))
            baseName = str(self.appDirPath / appName)
            self.stylesheetFileName = baseName + '.qss'
            self.stylesheetVarsFileName = baseName + 'Vars.pickle'

    def apply(self):
        """
        Apply the application window stylesheet, including variable value substitution
        """
        # This means:
        # 1) Read it from a file
        # 2) Replace all '{' and '}' which are QSS syntax (e.g. 'QWidget {') with 
        #    '{{' and '}}'. This protects the QSS syntax during the next steps.
        # 3) Replace all $word with {word} thus turning the string into a Python
        #    string with argument specifiers (e.g. '{main_font_family}'). 
        # 4) Replace all the format-string argument specifiers with variables
        # 5) Apply the resulting stylesheet to the App

        #print ( "apply: {}".format(self.stylesheetFileName))
        stylesheetText = None

        try:
            with open(self.stylesheetFileName, 'r') as content_file:
                stylesheetText = content_file.read()
        except:
            pass

        if not stylesheetText:
            print(f'No file {self.stylesheetFileName}')
            stylesheetText = self.defaultStylesheet
            try:
                with open(self.stylesheetFileName, 'w') as content_file:
                    content_file.write(stylesheetText)
            except:
                print(f'Could not write default stylesheet file {self.stylesheetFileName}')
                

        # These next two could be done in one pass using the cool multiple_replace() from
        # https://stackoverflow.com/questions/15175142/how-can-i-do-multiple-substitutions-using-regex-in-python
        # But this is easier to read
        stylesheetText = stylesheetText.replace('{', '{{')
        stylesheetText = stylesheetText.replace('}', '}}')

        # Turn all $word into {word}
        stylesheetText = re.sub(r'\$(([a-z]|[A-Z])\w*)', r'{\1}',  stylesheetText)
        # for k, v in self.varsDict.items():
        #    print ( f'{k}: {v}' )

        # substitute everything from our variables dict
        result = stylesheetText.format_map(self.varsDict)

        # apply
        self.app.setStyleSheet(result)

    def save_stylesheet_vars(self ):
        """Write our variables dict out to a file"""
        with open(self.stylesheetVarsFileName, 'wb') as h:
            pickle.dump(self.varsDict, h)

    def set_missing_stylesheet_vars(self ):
        """Set all the missing variables in the variables dict to default values"""
        for k in self.defaultVarsDict:
            self.varsDict.setdefault(k, self.defaultVarsDict[k])

    def read_stylesheet_vars(self ):
        """Read all the variables from saved file into our dict"""
        try:
            with open(self.stylesheetVarsFileName, 'rb') as h:
                self.varsDict = pickle.loads(h.read())
        except FileNotFoundError as e:
            print(e)
        # Maybe some values are missing, fix it up
        self.set_missing_stylesheet_vars()

    def zoom_in(self):
        """Increase the value of variables that influence the size of the UI"""
        # Trim off 'pt' at the end of the string. Maybe a little fragile...
        fontSize = int(self.varsDict['main_font_size'][0:-2])
        fontSize += 1
        self.varsDict['main_font_size'] = f'{fontSize}pt'

        # Trim off 'px' at the end of the string. Also a little fragile...
        scrollBarWidth = int(self.varsDict['scroll_bar_width'][0:-2])
        scrollBarWidth += 1
        self.varsDict['scroll_bar_width'] = f'{scrollBarWidth}px'

    def zoom_out(self):
        """Decrease the value of variables that influence the size of the UI"""
        # Trim off 'pt' at the end of the string. Maybe a little fragile...
        fontSize = int(self.varsDict['main_font_size'][0:-2])
        if fontSize > 0:
            fontSize -= 1
            self.varsDict['main_font_size'] = f'{fontSize}pt'

        # Trim off 'px' at the end of the string. Also a little fragile...
        scrollBarWidth = int(self.varsDict['scroll_bar_width'][0:-2])
        if scrollBarWidth > 0:
            scrollBarWidth -= 1
            self.varsDict['scroll_bar_width'] = f'{scrollBarWidth}px'

    # Variable setters / Properties - don't want to keep adding these for every variable
    # and the way it is evolving, it seems like we don't have to
    def set_main_font_family(self, family):
        self.varsDict['main_font_family'] = family

    def set_main_font_weight(self, weight):
        self.varsDict['main_font_weight'] = weight

    def set_main_font_size(self, size):
        self.varsDict['main_font_size'] = size

## Test Code
import argparse, sys
from   PyQt5.QtWidgets import QApplication, QVBoxLayout, QMainWindow

class TestAppMainWindow(QMainWindow):

    def __init__(self, parsedArgs, **kwargs ):
        super(TestAppMainWindow, self).__init__()
        self.setup_ui()
        self.show()

    def setup_ui(self):
        vbox = QVBoxLayout(self.centralWidget())

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-w', '--test-write-vars', action='store_const', const=True,
                        help='Test writing variables to file')
    parser.add_argument('-r', '--test-read-vars', action='store_const', const=True,
                        help='Test reading variables from file')
    parsedArgs,unparsedArgs = parser.parse_known_args()

    organizationName='Craton'
    appName='StyMgrTest'

    # Pass unparsed args to Qt, might have some X Windows args, like --display
    qtArgs = sys.argv[:1] + unparsedArgs
    app = QApplication(qtArgs)

    settings = QSettings(organizationName, appName)

    styMgr = StylesheetManager(app, settings, appName)

    if parsedArgs.test_write_vars:
        print('write')
        styMgr.save_stylesheet_vars()
        sys.exit(0)

    if parsedArgs.test_read_vars:
        print('read')
        styMgr.read_stylesheet_vars()
        for k, v in styMgr.varsDict.items():
            print(k, v)
        sys.exit(0)

    mainw = TestAppMainWindow(parsedArgs, app=app, organizationName=organizationName, appName=appName)
    
    sys.exit(app.exec_())



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="uqtie",
    version="0.1.2",
    author="Nik Langrind",
    author_email="langrind@gmail.com",
    description="Utilities for small or throwaway PyQt5 applications",
    license='MIT',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/langrind/uqtie",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    scripts=['example/simple_uqtie.py'],
)
